
' Region Description
'
' Name: ADRepCheckLatency-Detect
' Author: Nils Kaczenski
' Version: see code
' Description: see Readme-Howto.txt
' inspired by a design by Florian Frommherz 
' 
' EndRegion

' Script name
strTitel = "ADRepCheckLatency-Detect 1.0"

' have we been called by cscript?
If InStr(1, WScript.FullName, "cscript", vbTextCompare) = 0 Then 
	' no - tell user to do so
	InputBox "Please run this script from a CMD window using the following command line." _
	& VbCrLf & "You need read permission to Active Directory objects to run this script.", _
	strTitel, "cscript " & """" & WScript.ScriptFullName & """"
	WScript.Quit
End If

' read parameters from external script file
strParametersFileName = Replace(LCase(WScript.ScriptFullName), "detect", "parameters")
Include strParametersFileName

' fetch CMD arguments
If WScript.Arguments.Count = 1 Then
	strArg = WScript.Arguments(0)
Else
	WScript.Echo "Note: If you want to automate the script please specify the name"
	WScript.Echo "of the remote DC as a command-line argument." & VbCrLf
	strArg = InputBox ("Which remote DC do you want to query?", strTitel)
End If

strRemoteDC = strArg

WScript.Echo strTitel & ": Started at " & Now & " (local time)"
WScript.Echo VbCrLf & "Retrieving data from AD ..." & VbCrLf

' bind to "local" object
Set objChangeLocal = GetObject("LDAP://" & strLocalDC & "/" & strObject)

' bind to "remote" object
Set objChangeRemote = GetObject("LDAP://" & strRemoteDC & "/" & strObject)

' retrieve local data
dtmWhenCreated = objChangeLocal.Get("whenCreated")
dtmLocalWhenChanged = objChangeLocal.Get("whenChanged")
' attribute could be empty, so catch errors
strLocalValue = ""
On Error Resume Next
strLocalValue = objChangeLocal.Get(strAttribute)
On Error Goto 0
If strLocalValue = "" Then strLocalValue = "<none>"

WScript.Echo "Data on Local DC"
WScript.Echo "----------------"
WScript.Echo "Local DC: " & strLocalDC
WScript.Echo "Object: " & strObject
WScript.Echo "Object created: " & dtmWhenCreated
WScript.Echo "Object changed locally: " & dtmLocalWhenChanged
WScript.Echo "Attribute to query: " & strAttribute
WScript.Echo "Current value: " & strLocalValue & VbCrLf

blnDetected = False

' retrieve remote data
Do
	objChangeRemote.GetInfo
	dtmWhenCreated = objChangeRemote.Get("whenCreated")
	dtmRemoteWhenChanged = objChangeRemote.Get("whenChanged")
	' attribute could be empty, so catch errors
	strRemoteValue = ""
	On Error Resume Next
	strRemoteValue = objChangeRemote.Get(strAttribute)
	On Error Goto 0
	If strRemoteValue = "" Then strRemoteValue = "<none>"
	
	If strRemoteValue = strLocalValue Then
		' change detected - continue!
		blnDetected = True
		WScript.Echo "Change detected on " & strRemoteDC & " at " & Now & " (local time)" & VbCrLf
	Else
		' change not yet here - wait a minute and repeat
		WScript.Echo "... waiting to detect for one more minute (local time: " & Now & ")"
		WScript.Sleep 1000*60
	End If
Loop Until blnDetected


WScript.Echo "Data on Remote DC"
WScript.Echo "-----------------"
WScript.Echo "Remote DC: " & strRemoteDC
WScript.Echo "Object: " & strObject
WScript.Echo "Object created: " & dtmWhenCreated
WScript.Echo "Object changed remotely: " & dtmRemoteWhenChanged
WScript.Echo "Attribute to query: " & strAttribute
WScript.Echo "Current value: " & strRemoteValue

WScript.Echo VbCrLf & "*** The replication latency between " & strLocalDC & " and " & strRemoteDC & " is: " _
	& Abs(DateDiff("s", dtmRemoteWhenChanged, dtmLocalWhenChanged)) & " seconds."
WScript.Echo VbCrLf & strTitel & ": Finished at " & Now & " (local time)"

' VBScript "Include" routine
Sub Include(sInstFile)
	On Error Resume Next

	Dim oFSO, f, s

	Set oFSO = CreateObject("Scripting.FileSystemObject")
	If oFSO.FileExists(sInstFile) Then
		Set f = oFSO.OpenTextFile(sInstFile)
		s = f.ReadAll
		f.Close
		ExecuteGlobal s
	End If

	Set oFSO = Nothing
	Set f = Nothing
End Sub
